@@ -19,11 +19,11 @@ import time
 from collections import defaultdict
 
 from httpx import Timeout, URL, Client, AsyncClient
-from typing import Dict, Tuple
+from typing import Dict, Tuple, Optional
 
 from byteplussdkcore.rest import ApiException
 from ._exceptions import ArkAPIError
-
+from ._models import BaseModel
 import byteplussdkark
 
 from . import resources
@@ -37,11 +37,12 @@ from ._constants import (
     _DEFAULT_MANDATORY_REFRESH_TIMEOUT,
     _DEFAULT_STS_TIMEOUT,
     _DEFAULT_RESOURCE_TYPE,
+    _PRESETENDPOINT_RESOURCE_TYPE,
     DEFAULT_TIMEOUT,
 )
 from ._streaming import Stream
 
-from ._utils._key_agreement import key_agreement_client
+from ._utils._key_agreement import key_agreement_client, get_cert_info
 from ._utils._model_breaker import ModelBreaker
 
 __all__ = ["Ark", "AsyncArk"]
@@ -62,6 +63,7 @@ class Ark(SyncAPIClient):
     batch: batch.Batch
     model_breaker_map: dict[str, ModelBreaker]
     model_breaker_lock: threading.Lock
+    files: resources.Files
 
     def __init__(
         self,
@@ -70,7 +72,7 @@ class Ark(SyncAPIClient):
         ak: str | None = None,
         sk: str | None = None,
         api_key: str | None = None,
-        region: str = "ap-southeast-1",
+        region: str = "cn-beijing",
         timeout: float | Timeout | None = DEFAULT_TIMEOUT,
         max_retries: int = DEFAULT_MAX_RETRIES,
         http_client: Client | None = None,
@@ -121,24 +123,30 @@ class Ark(SyncAPIClient):
         self.embeddings = resources.Embeddings(self)
         self.context = resources.Context(self)
         self.multimodal_embeddings = resources.MultimodalEmbeddings(self)
-        self.batch_chat = resources.BatchChat(self)
-        self.batch = batch.Batch(self)
         self.content_generation = resources.ContentGeneration(self)
         self.images = resources.Images(self)
+        self.batch_chat = resources.BatchChat(self)
         self.responses = resources.Responses(self)
         self.input_items = resources.InputItems(self)
+        self.batch = batch.Batch(self)
         self.model_breaker_map = defaultdict(ModelBreaker)
         self.model_breaker_lock = threading.Lock()
+        self.files = resources.Files(self)
         # self.classification = resources.Classification(self)
 
-    def _get_endpoint_sts_token(self, endpoint_id: str):
+    def _get_endpoint_sts_token(self, endpoint_id: str, project_name: str = None):
         if self._sts_token_manager is None:
             if self.ak is None or self.sk is None:
                 raise ArkAPIError("must set ak and sk before get endpoint token.")
             self._sts_token_manager = StsTokenManager(self.ak, self.sk, self.region)
-        return self._sts_token_manager.get(endpoint_id)
+        resource_type: str = self.get_resource_type_by_endpoint_id(endpoint_id)
+        if resource_type == _PRESETENDPOINT_RESOURCE_TYPE and (project_name is None or project_name.strip() == ""):
+            raise ArkAPIError("must set project_name when get preset endpoint token.")
+        return self._sts_token_manager.get(endpoint_id, resource_type=resource_type, project_name=project_name)
 
-    def _get_endpoint_certificate(self, endpoint_id: str) -> key_agreement_client:
+    def _get_endpoint_certificate(
+        self, endpoint_id: str
+    ) -> Tuple[key_agreement_client, str, str, float]:
         if self._certificate_manager is None:
             cert_path = os.environ.get("E2E_CERTIFICATE_PATH")
             if (
@@ -155,13 +163,6 @@ class Ark(SyncAPIClient):
             )
         return self._certificate_manager.get(endpoint_id)
 
-    def _get_bot_sts_token(self, bot_id: str):
-        if self._sts_token_manager is None:
-            if self.ak is None or self.sk is None:
-                raise ArkAPIError("must set ak and sk before get endpoint token.")
-            self._sts_token_manager = StsTokenManager(self.ak, self.sk, self.region)
-        return self._sts_token_manager.get(bot_id, resource_type="bot")
-
     @property
     def auth_headers(self) -> dict[str, str]:
         api_key = self.api_key
@@ -171,22 +172,31 @@ class Ark(SyncAPIClient):
         with self.model_breaker_lock:
             return self.model_breaker_map[model_name]
 
+    def get_resource_type_by_endpoint_id(self, endpoint_id: str) -> str:
+        if endpoint_id.startswith("ep-m-"):
+            return _PRESETENDPOINT_RESOURCE_TYPE
+        if endpoint_id.startswith("ep-"):
+            return _DEFAULT_RESOURCE_TYPE
+        # for model id, default to preset endpoint
+        return _PRESETENDPOINT_RESOURCE_TYPE
+
 
 class AsyncArk(AsyncAPIClient):
-    beta: beta.Beta
+    beta: beta.AsyncBeta
     chat: resources.AsyncChat
     embeddings: resources.AsyncEmbeddings
     context: resources.AsyncContext
     multimodal_embeddings: resources.AsyncMultimodalEmbeddings
+    content_generation: resources.AsyncContentGeneration
+    images: resources.AsyncImages
     batch_chat: resources.AsyncBatchChat
     responses: resources.AsyncResponses
     input_items: resources.AsyncInputItems
     """ `batch_chat` is deprecated, use `batch.chat` instead """
     batch: batch.AsyncBatch
-    content_generation: resources.AsyncContentGeneration
-    images: resources.AsyncImages
     model_breaker_map: dict[str, ModelBreaker]
     model_breaker_lock: asyncio.Lock
+    files: resources.AsyncFiles
 
     def __init__(
         self,
@@ -195,7 +205,7 @@ class AsyncArk(AsyncAPIClient):
         sk: str | None = None,
         api_key: str | None = None,
         base_url: str | URL = BASE_URL,
-        region: str = "ap-southeast-1",
+        region: str = "cn-beijing",
         timeout: float | Timeout | None = DEFAULT_TIMEOUT,
         max_retries: int = DEFAULT_MAX_RETRIES,
         http_client: AsyncClient | None = None,
@@ -246,14 +256,15 @@ class AsyncArk(AsyncAPIClient):
         self.embeddings = resources.AsyncEmbeddings(self)
         self.context = resources.AsyncContext(self)
         self.multimodal_embeddings = resources.AsyncMultimodalEmbeddings(self)
-        self.batch_chat = resources.AsyncBatchChat(self)
-        self.batch = batch.AsyncBatch(self)
         self.content_generation = resources.AsyncContentGeneration(self)
         self.images = resources.AsyncImages(self)
+        self.batch_chat = resources.AsyncBatchChat(self)
         self.responses = resources.AsyncResponses(self)
         self.input_items = resources.AsyncInputItems(self)
+        self.batch = batch.AsyncBatch(self)
         self.model_breaker_map = defaultdict(ModelBreaker)
         self.model_breaker_lock = asyncio.Lock()
+        self.files = resources.AsyncFiles(self)
         # self.classification = resources.AsyncClassification(self)
 
     def _get_endpoint_sts_token(self, endpoint_id: str):
@@ -263,7 +274,9 @@ class AsyncArk(AsyncAPIClient):
             self._sts_token_manager = StsTokenManager(self.ak, self.sk, self.region)
         return self._sts_token_manager.get(endpoint_id)
 
-    def _get_endpoint_certificate(self, endpoint_id: str) -> key_agreement_client:
+    def _get_endpoint_certificate(
+        self, endpoint_id: str
+    ) -> Tuple[key_agreement_client, str, str, float]:
         if self._certificate_manager is None:
             cert_path = os.environ.get("E2E_CERTIFICATE_PATH")
             if (
@@ -327,6 +340,7 @@ class StsTokenManager(object):
         ttl: int = _DEFAULT_STS_TIMEOUT,
         is_mandatory: bool = False,
         resource_type: str = _DEFAULT_RESOURCE_TYPE,
+        project_name: str = None,
     ):
         if ttl < self._advisory_refresh_timeout * 2:
             raise ArkAPIError(
@@ -337,7 +351,7 @@ class StsTokenManager(object):
 
         try:
             api_key, expired_time = self._load_api_key(
-                ep, ttl, resource_type=resource_type
+                ep, ttl, resource_type=resource_type, project_name=project_name
             )
             self._endpoint_sts_tokens[ep] = (api_key, expired_time)
         except ApiException as e:
@@ -346,7 +360,7 @@ class StsTokenManager(object):
             else:
                 logging.error("load api key cause error: e={}".format(e))
 
-    def _refresh(self, ep: str, resource_type: str = _DEFAULT_RESOURCE_TYPE):
+    def _refresh(self, ep: str, resource_type: str = _DEFAULT_RESOURCE_TYPE, project_name: str = None):
         if not self._need_refresh(ep, self._advisory_refresh_timeout):
             return
 
@@ -360,7 +374,7 @@ class StsTokenManager(object):
                 )
 
                 self._protected_refresh(
-                    ep, is_mandatory=is_mandatory_refresh, resource_type=resource_type
+                    ep, is_mandatory=is_mandatory_refresh, resource_type=resource_type, project_name=project_name
                 )
                 return
             finally:
@@ -371,11 +385,11 @@ class StsTokenManager(object):
                     return
 
                 self._protected_refresh(
-                    ep, is_mandatory=True, resource_type=resource_type
+                    ep, is_mandatory=True, resource_type=resource_type, project_name=project_name
                 )
 
-    def get(self, ep: str, resource_type: str = _DEFAULT_RESOURCE_TYPE) -> str:
-        self._refresh(ep, resource_type=resource_type)
+    def get(self, ep: str, resource_type: str = _DEFAULT_RESOURCE_TYPE, project_name: str = None) -> str:
+        self._refresh(ep, resource_type=resource_type, project_name=project_name)
         return self._endpoint_sts_tokens[ep][0]
 
     def _load_api_key(
@@ -383,12 +397,15 @@ class StsTokenManager(object):
         ep: str,
         duration_seconds: int,
         resource_type: str = _DEFAULT_RESOURCE_TYPE,
+        project_name: str = None,
     ) -> Tuple[str, int]:
         get_api_key_request = byteplussdkark.GetApiKeyRequest(
             duration_seconds=duration_seconds,
             resource_type=resource_type,
             resource_ids=[ep],
         )
+        if project_name is not None and project_name.strip() != "":
+            get_api_key_request.project_name = project_name
         resp: byteplussdkark.GetApiKeyResponse = self.api_instance.get_api_key(
             get_api_key_request
         )
@@ -396,12 +413,15 @@ class StsTokenManager(object):
         return resp.api_key, resp.expired_time
 
 
+class CertificateResponse(BaseModel):
+    error: Optional[Dict[str, str]] = None
+    """The error information."""
+
+    Certificate: str
+    """The certificate content."""
+
+
 class E2ECertificateManager(object):
-
-    class CertificateResponse:
-        Certificate: str
-        """The certificate content."""
-
     def __init__(
         self,
         ak: str,
@@ -410,7 +430,9 @@ class E2ECertificateManager(object):
         base_url: str | URL = BASE_URL,
         api_key: str | None = None,
     ):
-        self._certificate_manager: Dict[str, key_agreement_client] = {}
+        self._certificate_manager: Dict[
+            str, Tuple[key_agreement_client, str, str, float]
+        ] = {}
 
         # local cache prepare
         self._init_local_cert_cache()
@@ -441,15 +463,23 @@ class E2ECertificateManager(object):
         self._e2e_uri = "/e2e/get/certificate"
         self._x_session_token = {"X-Session-Token": self._e2e_uri}
 
+        self._aicc_enabled = False
+        if os.environ.get("VOLC_ARK_ENCRYPTION") == "AICC":
+            self._aicc_enabled = True
+
     def _load_cert_by_cert_path(self) -> str:
         with open(self.cert_path, "r") as f:
             cert_pem = f.read()
         return cert_pem
 
     def _load_cert_by_ak_sk(self, ep: str) -> str:
-        get_endpoint_certificate_request = byteplussdkark.GetEndpointCertificateRequest(
-            id=ep
+        get_endpoint_certificate_request = (
+            byteplussdkark.GetEndpointCertificateRequest(id=ep)
         )
+        if self._aicc_enabled:
+            get_endpoint_certificate_request = (
+                byteplussdkark.GetEndpointCertificateRequest(id=ep, type="AICCv0.1")
+            )
         try:
             resp: byteplussdkark.GetEndpointCertificateResponse = (
                 self.api_instance.get_endpoint_certificate(
@@ -465,17 +495,20 @@ class E2ECertificateManager(object):
 
     def _sync_load_cert_by_auth(self, ep: str) -> str:
         try:  # try to make request with session header (used for header statistic)
+            req_body = {"model": ep}
+            if self._aicc_enabled:
+                req_body["type"] = "AICCv0.1"
             resp = self.client.post(
                 self._e2e_uri,
                 options={"headers": self._x_session_token},
-                body={"model": ep},
-                cast_to=self.CertificateResponse,
+                body=req_body,
+                cast_to=CertificateResponse,
             )
         except Exception as e:
             raise ArkAPIError("Getting Certificate failed: %s\n" % e)
-        if "error" in resp:
-            raise ArkAPIError("Getting Certificate failed: %s\n" % resp["error"])
-        return resp["Certificate"]
+        if resp.error is not None:
+            raise ArkAPIError("Getting Certificate failed: %s\n" % resp.error)
+        return resp.Certificate
 
     def _save_cert_to_file(self, ep: str, cert_pem: str):
         cert_file_path = os.path.join(self._cert_storage_path, f"{ep}.pem")
@@ -489,20 +522,34 @@ class E2ECertificateManager(object):
             current_time = time.time()
             time_difference = current_time - last_modified_time
             if time_difference <= self._cert_expiration_seconds:
+                cert_pem = None
                 with open(cert_file_path, "r") as f:
-                    return f.read()
-            else:
-                os.remove(cert_file_path)
+                    cert_pem = f.read()
+                ring, key, _ = get_cert_info(cert_pem)
+                # check cert is complement with AICC/PCA
+                if (ring == "" or key == "") and not self._aicc_enabled:
+                    return cert_pem
+                if ring != "" and key != "" and self._aicc_enabled:
+                    return cert_pem
+            os.remove(cert_file_path)
         return None
 
     def _init_local_cert_cache(self):
-        self._cert_storage_path = "/tmp/ark/certificates"
+        self._cert_storage_path = os.path.expanduser("~/.ark/certificates")
         self._cert_expiration_seconds = 14 * 24 * 60 * 60  # 14 days
 
         if not os.path.exists(self._cert_storage_path):
-            os.makedirs(self._cert_storage_path)
+            try:
+                os.makedirs(self._cert_storage_path)
+            except FileExistsError:
+                pass
+            except Exception as e:
+                raise ArkAPIError(
+                    "failed to create certificate directory %s: %s\n"
+                    % (self._cert_storage_path, e)
+                )
 
-    def get(self, ep: str) -> key_agreement_client:
+    def get(self, ep: str) -> Tuple[key_agreement_client, str, str, float]:
         if ep not in self._certificate_manager:
             cert_pem = self._load_cert_locally(ep)
             if cert_pem is None:
@@ -513,7 +560,11 @@ class E2ECertificateManager(object):
                 else:
                     cert_pem = self._load_cert_by_ak_sk(ep)
                 self._save_cert_to_file(ep, cert_pem)
-            self._certificate_manager[ep] = key_agreement_client(
-                certificate_pem_string=cert_pem
+            ring, key, exp_time = get_cert_info(cert_pem)
+            self._certificate_manager[ep] = (
+                key_agreement_client(certificate_pem_string=cert_pem),
+                ring,
+                key,
+                exp_time,
             )
         return self._certificate_manager[ep]
